package cn.com.qmth.examcloud.core.basic.api.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelError;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.dao.SpecialtyRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatlyRelationEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.SpecialtyEntity;
import cn.com.qmth.examcloud.core.basic.service.impl.SpecialtyServiceImpl;
import io.swagger.annotations.ApiOperation;

/**
 * @Description: 专业服务API
 * @author weiwenhai
 * @Date 2017年8月7号
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/specialty")
public class SpecialtyController extends ControllerSupport {

	@Autowired
	SpecialtyServiceImpl specialtyService;

	@Autowired
	SpecialtyRepo specialtyRepo;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param curPage
	 * @param pageSize
	 * @param name
	 * @param code
	 * @param enable
	 * @param courseId
	 * @return
	 */
	@ApiOperation(value = "分页查询专业")
	@GetMapping("specialtyPage/{curPage}/{pageSize}")
	public Page<SpecialtyEntity> getSpecialtyPage(@PathVariable Integer curPage,
			@PathVariable Integer pageSize, @RequestParam(required = false) String name,
			@RequestParam(required = false) String code,
			@RequestParam(required = false) Boolean enable,
			@RequestParam(required = false) Long courseId) {
		User accessUser = getAccessUser();

		Specification<SpecialtyEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();

			predicates.add(cb.equal(root.get("rootOrgId"), accessUser.getRootOrgId()));

			if (StringUtils.isNotBlank(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotBlank(code)) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(code)));
			}
			if (null != enable) {
				predicates.add(cb.equal(root.get("enable"), enable));
			}

			if (null != courseId) {
				Subquery<CourseSpeciatlyRelationEntity> subquery = query
						.subquery(CourseSpeciatlyRelationEntity.class);
				Root<CourseSpeciatlyRelationEntity> subRoot = subquery
						.from(CourseSpeciatlyRelationEntity.class);
				subquery.select(subRoot.get("specialtyId"));
				Predicate p1 = cb.equal(subRoot.get("courseId"), courseId);
				Predicate p2 = cb.equal(subRoot.get("specialtyId"), root.get("id"));
				subquery.where(cb.and(p1, p2));
				predicates.add(cb.exists(subquery));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = new PageRequest(curPage, pageSize,
				new Sort(Direction.DESC, "updateTime"));

		Page<SpecialtyEntity> page = specialtyRepo.findAll(specification, pageRequest);
		return page;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @return
	 */
	@ApiOperation(value = "查询所有专业")
	@GetMapping("all")
	public List<SpecialtyEntity> getAll() {
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();
		List<SpecialtyEntity> list = specialtyRepo.findByRootOrgId(rootOrgId);
		return list;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param orgId
	 * @param code
	 * @return
	 */
	@ApiOperation(value = "查询专业")
	@GetMapping("byCode")
	public SpecialtyEntity getByCode(@RequestParam Long orgId, @RequestParam String code) {
		SpecialtyEntity specialty = specialtyRepo.findByRootOrgIdAndCode(orgId, code);
		if (null == specialty) {
			throw new StatusException("B-620002", "专业不存在");
		}
		validateRootOrgIsolation(specialty.getRootOrgId());
		return specialty;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 */
	@ApiOperation(value = "删除专业")
	@DeleteMapping("{ids}")
	public void delete(@PathVariable String ids) {
		List<Long> specialtyIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long specialtyId : specialtyIds) {
			SpecialtyEntity one = specialtyRepo.findOne(specialtyId);
			if (null == one) {
				continue;
			}
			validateRootOrgIsolation(one.getRootOrgId());
			specialtyRepo.delete(one);
		}
	}

	@ApiOperation(value = "下载导入模板", notes = "下载导入模板")
	@GetMapping("downloadTemplate")
	public void getDownloadTemplate(HttpServletResponse response) {
		// List<SpecialtyDto> list = new ArrayList<SpecialtyDto>();
		// ExportService.exportEXCEL("专业导入模板", SpecialtyDto.class, list,
		// response);

	}

	@ApiOperation(value = "导入", notes = "导入")
	@PostMapping("/import")
	public List<ExcelError> importCourse(HttpServletRequest request,
			@RequestParam CommonsMultipartFile file) throws IOException {
		// cn.com.qmth.examcloud.commons.web.security.bean.User accessUser =
		// getAccessUser();
		// Long orgId = accessUser.getRootOrgId();
		//
		// List<ExcelError> errors = specialtyService.importSpecialty(orgId,
		// file.getInputStream());
		// return errors;
		return null;
	}

	/**
	 * 导出专业
	 * 
	 * @param specialty
	 * @param response
	 */
	@ApiOperation(value = "导出专业", notes = "导出专业")
	@GetMapping("export")
	public void exportSpecialty(@ModelAttribute SpecialtyEntity specialty,
			HttpServletResponse response) {
		// List<SpecialtyDto> list = new ArrayList<SpecialtyDto>();
		// specialtyService.findAll(specialty).forEach(c -> {
		// list.add(new SpecialtyDto(c));
		// });
		// ExportService.exportEXCEL("专业列表", SpecialtyDto.class, list,
		// response);
	}
}
