package cn.com.qmth.examcloud.core.basic.api.provider;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCourseReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveCourseReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCourseResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveCourseResp;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.enums.CourseLevel;
import io.swagger.annotations.ApiOperation;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年7月10日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.cloud.basic}" + "course")
public class CourseCloudServiceProvider implements CourseCloudService {

	private static final long serialVersionUID = -2733647860311223743L;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	CourseRepo courseRepo;

	@ApiOperation(value = "保存课程")
	@PostMapping("saveCourse")
	@Override
	public SaveCourseResp saveCourse(@RequestBody SaveCourseReq courseReq) {

		Long rootOrgId = courseReq.getRootOrgId();
		if (null == rootOrgId) {
			throw new StatusException("B-160000", "rootOrgId is null");
		}
		OrgEntity rootOrg = orgRepo.findOne(rootOrgId);
		if (null == rootOrg) {
			throw new StatusException("B-160001", "机构不存在");
		}
		if (null != rootOrg.getParentId()) {
			throw new StatusException("B-160002", "机构错误");
		}

		CourseEntity course = null;

		Long courseId = courseReq.getCourseId();
		String courseCode = courseReq.getCourseCode();

		if (null != courseId) {
			course = courseRepo.findOne(courseId);

			if (null == course) {
				throw new StatusException("B-160006", "课程ID错误");
			}

			if (!rootOrgId.equals(course.getRootOrgId())) {
				throw new StatusException("B-160006", "课程机构错误");
			}

		} else if (StringUtils.isNotBlank(courseCode)) {

			course = courseRepo.findByRootOrgIdAndCode(rootOrgId, courseCode);
			if (null != course) {
				if (null == course.getEnable()) {
					course.setEnable(true);
				}
				if (null == course.getLevel()) {
					course.setLevel(CourseLevel.ALL);
				}
			} else {
				course = new CourseEntity();
				course.setRootOrgId(courseReq.getRootOrgId());
				course.setCode(courseReq.getCourseCode());
				course.setEnable(true);
				course.setLevel(CourseLevel.ALL);
			}

		} else {
			throw new StatusException("B-160004", "courseId,courseCode can not be all null");
		}

		if (null != courseReq.getEnable()) {
			course.setEnable(courseReq.getEnable());
		}

		if (null != courseReq.getCourseName()) {
			course.setName(courseReq.getCourseName());
		}

		if (null != courseReq.getCourseLevel()) {
			CourseLevel courseLevel = CourseLevel.getCourseLevel(courseReq.getCourseLevel());
			course.setLevel(courseLevel);
		}

		long updateTime = 0L;
		if (null != course.getUpdateTime()) {
			updateTime = course.getUpdateTime().getTime();
		}

		CourseEntity saved = courseRepo.saveAndFlush(course);

		// 同步操作
		if (updateTime != saved.getUpdateTime().getTime()) {

		}

		SaveCourseResp resp = new SaveCourseResp();
		CourseBean courseBean = new CourseBean();
		courseBean.setId(saved.getId());
		courseBean.setCode(saved.getCode());
		courseBean.setLevel(saved.getLevel().name());
		courseBean.setName(saved.getName());
		courseBean.setRootOrgId(saved.getRootOrgId());

		resp.setCourseBean(courseBean);

		return resp;
	}

	@ApiOperation(value = "查询课程")
	@PostMapping("getCourse")
	@Override
	public GetCourseResp getCourse(@RequestBody GetCourseReq req) {
		GetCourseResp resp = new GetCourseResp();
		Long rootOrgId = req.getRootOrgId();
		String code = req.getCode();
		Long id = req.getId();

		if (null == rootOrgId) {
			throw new StatusException("B-160000", "rootOrgId is null");
		}
		OrgEntity rootOrg = orgRepo.findOne(rootOrgId);
		if (null == rootOrg) {
			throw new StatusException("B-160001", "机构不存在");
		}
		if (null != rootOrg.getParentId()) {
			throw new StatusException("B-160002", "机构错误");
		}

		CourseEntity c = null;
		if (null != id) {
			c = courseRepo.findOne(id);

		} else if (StringUtils.isNotBlank(code)) {
			c = courseRepo.findByRootOrgIdAndCode(rootOrgId, code);
		} else {
			throw new StatusException("B-160002", "id,code can not be all null");
		}

		if (null == c) {
			throw new StatusException("B-160003", "课程不存在");
		}

		CourseBean courseBean = new CourseBean();
		courseBean.setId(c.getId());
		courseBean.setCode(c.getCode());
		courseBean.setLevel(c.getLevel().name());
		courseBean.setName(c.getName());
		courseBean.setRootOrgId(c.getRootOrgId());
		courseBean.setEnable(c.getEnable());
		resp.setCourseBean(courseBean);

		return resp;
	}

}
