package cn.com.qmth.examcloud.core.basic.api.provider;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveOrgReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveOrgResp;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgInfo;
import cn.com.qmth.examcloud.core.basic.service.impl.OrgServiceImpl;
import io.swagger.annotations.ApiOperation;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年8月23日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.cloud.basic}" + "org")
public class OrgCloudServiceProvider extends ControllerSupport implements OrgCloudService {

	private static final long serialVersionUID = -7858439296389761341L;

	@Autowired
	private OrgServiceImpl orgService;

	@Autowired
	private OrgRepo orgRepo;

	@ApiOperation(value = "保存机构")
	@PostMapping("saveOrg")
	@Override
	public SaveOrgResp saveOrg(@RequestBody SaveOrgReq req) {
		OrgInfo info = new OrgInfo();
		info.setCode(req.getCode());
		info.setName(req.getName());
		info.setContacts(req.getContacts());
		info.setTelephone(req.getTelephone());
		info.setDomainName(req.getDomainName());
		info.setRemark(req.getRemark());

		OrgEntity saved = orgService.saveRootOrg(info);

		SaveOrgResp orgResp = new SaveOrgResp();
		orgResp.setId(saved.getId());
		orgResp.setName(saved.getName());
		return orgResp;
	}

	@ApiOperation(value = "按机构名称模糊查询机构列表")
	@PostMapping("getOrg")
	@Override
	public GetOrgResp getOrg(@RequestBody GetOrgReq req) {
		Long orgId = req.getOrgId();
		String orgCode = req.getOrgCode();
		Long rootOrgId = req.getRootOrgId();

		if (null == orgId && StringUtils.isBlank(orgCode)) {
			throw new StatusException("B-150000", "orgId,orgCode不能都为空");
		}

		if (null != orgId && StringUtils.isNotBlank(orgCode)) {
			throw new StatusException("B-150001", "orgId,orgCode不能都不为空");
		}

		OrgEntity org = null;
		if (null != orgId) {
			org = orgRepo.findOne(orgId);
		} else if (StringUtils.isNotBlank(orgCode)) {
			if (null == rootOrgId) {
				throw new StatusException("B-150001", "rootOrgId is null");
			}
			org = orgRepo.findByRootIdAndCode(rootOrgId, orgCode);
		}

		if (null == org) {
			throw new StatusException("B-150001", "机构不存在");
		}

		OrgBean orgBean = new OrgBean();
		orgBean.setId(org.getId());
		orgBean.setName(org.getName());
		orgBean.setCode(org.getCode());
		orgBean.setParentId(org.getParentId());
		orgBean.setRootId(org.getRootId());
		orgBean.setEnable(org.getEnable());

		GetOrgResp resp = new GetOrgResp();
		resp.setOrg(orgBean);
		return resp;
	}
}
