package cn.com.qmth.examcloud.core.basic.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.enums.CourseLevel;
import cn.com.qmth.examcloud.core.basic.service.CourseService;
import cn.com.qmth.examcloud.core.basic.service.bean.CourseInfo;

@Service
public class CourseServiceImpl implements CourseService {

	@Autowired
	CourseRepo courseRepo;

	@Autowired
	OrgRepo orgRepo;

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.basic.service.CourseService#saveCourse(cn.com.
	 * qmth.examcloud.core.basic.service.bean.CourseInfo)
	 */
	@Override
	public CourseEntity saveCourse(CourseInfo courseInfo) {
		Long rootOrgId = courseInfo.getRootOrgId();
		Long courseId = courseInfo.getId();
		String courseCode = courseInfo.getCode();
		CourseLevel level = courseInfo.getLevel();
		Boolean enable = courseInfo.getEnable();
		String name = courseInfo.getName();

		if (null == rootOrgId) {
			throw new StatusException("B-160000", "rootOrgId is null");
		}
		OrgEntity rootOrg = orgRepo.findOne(rootOrgId);
		if (null == rootOrg) {
			throw new StatusException("B-160001", "机构不存在");
		}
		if (null != rootOrg.getParentId()) {
			throw new StatusException("B-160002", "机构错误");
		}

		CourseEntity course = null;

		if (null != courseId) {
			course = courseRepo.findOne(courseId);

			if (null == course) {
				throw new StatusException("B-160006", "课程ID错误");
			}

			if (!rootOrgId.equals(course.getRootOrgId())) {
				throw new StatusException("B-160006", "课程机构错误");
			}

		} else if (StringUtils.isNotBlank(courseCode)) {

			course = courseRepo.findByRootOrgIdAndCode(rootOrgId, courseCode);

			if (null != course) {
				if (null == course.getEnable()) {
					course.setEnable(true);
				}
				if (null == course.getLevel()) {
					course.setLevel(CourseLevel.ALL);
				}
			} else {
				course = new CourseEntity();
				course.setRootOrgId(rootOrgId);
				course.setCode(courseCode);
				course.setEnable(true);
				course.setLevel(CourseLevel.ALL);
			}

		} else {
			throw new StatusException("B-160004", "courseId,courseCode can not be all null");
		}

		if (null != enable) {
			course.setEnable(enable);
		}

		if (StringUtils.isNotBlank(name)) {
			course.setName(name);
		}

		if (null != level) {
			course.setLevel(level);
		}

		long updateTime = 0L;
		if (null != course.getUpdateTime()) {
			updateTime = course.getUpdateTime().getTime();
		}

		CourseEntity saved = courseRepo.saveAndFlush(course);

		// 同步操作
		if (updateTime != saved.getUpdateTime().getTime()) {

		}
		return saved;
	}

}
