package cn.com.qmth.examcloud.core.basic.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.CourseSpeciatlyRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.SpecialtyRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatlyRelationEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.SpecialtyEntity;

@Service
public class CourseSpeciatlyRelationServiceImpl {

	@Autowired
	CourseSpeciatlyRelationRepo courseSpeciatlyRepo;

	@Autowired
	CourseRepo courseRepo;

	@Autowired
	SpecialtyRepo specialtyRepo;

	/**
	 * 根据专业ID查询关联课程
	 * 
	 * @param speciatlyId
	 * @return
	 */
	public List<CourseEntity> getAllCoursesBySpeciatlyId(String speciatlyId) {
		// 根据专业id查询 专业课程
		List<CourseSpeciatlyRelationEntity> list = courseSpeciatlyRepo
				.findBySpecialtyId(Long.parseLong(speciatlyId));
		List<CourseEntity> courses = new ArrayList<CourseEntity>();
		if (list == null) {
			return null;
		}
		courses = getCoursesByCourseSpeciatly(list);
		return courses;
	}

	// 根据课程专业查询课程集合
	public List<CourseEntity> getCoursesByCourseSpeciatly(
			List<CourseSpeciatlyRelationEntity> list) {
		List<CourseEntity> courses = new ArrayList<CourseEntity>();
		for (CourseSpeciatlyRelationEntity courseSpeciatly : list) {
			CourseEntity course = courseRepo.findOne(courseSpeciatly.getCourseId());
			courses.add(course);
		}
		return courses;
	}

	/**
	 * 获取未关联的课程
	 * 
	 * @param courses
	 * @return
	 */
	public List<CourseEntity> getCoursesNotInSpeciatly(List<CourseEntity> courses) {
		List<CourseEntity> list = new ArrayList<CourseEntity>();
		if (courses == null || courses.size() < 1) {
			list = courseRepo.findAll();
			return list;
		}
		List<Long> ids = new ArrayList<Long>();
		for (CourseEntity course : courses) {
			ids.add(course.getId());
		}
		list = courseRepo.findByIdNotIn(ids);
		return list;
	}

	public void addCourseSpecialty(Long userId, List<String> courseIds, String speciallyId) {
		// 首先判断该专业是否有关联的课程
		List<CourseSpeciatlyRelationEntity> list = courseSpeciatlyRepo
				.findBySpecialtyId(Long.parseLong(speciallyId));
		if (list != null || list.size() > 0) {
			for (CourseSpeciatlyRelationEntity courseSpeciatly : list) {
				courseSpeciatlyRepo.delete(courseSpeciatly);
			}
		}
		// 保存新关联的课程
		for (String courseId : courseIds) {
			CourseSpeciatlyRelationEntity courseSpeciatly = new CourseSpeciatlyRelationEntity();
			courseSpeciatly.setCourseId(Long.parseLong(courseId));
			courseSpeciatly.setSpecialtyId(Long.parseLong(speciallyId));
			courseSpeciatly.setCreator(userId);
			courseSpeciatlyRepo.save(courseSpeciatly);
		}
	}

	public List<SpecialtyEntity> getAllSpecialtyByCourseId(Long courseId) {
		List<CourseSpeciatlyRelationEntity> list = courseSpeciatlyRepo.findByCourseId(courseId);
		List<SpecialtyEntity> specialties = new ArrayList<SpecialtyEntity>();
		for (CourseSpeciatlyRelationEntity courseSpeciatly : list) {
			SpecialtyEntity one = specialtyRepo.findOne(courseSpeciatly.getSpecialtyId());
			if (null != one) {
				specialties.add(one);
			}
		}
		return specialties;
	}

	public void addCourse(Long userId, List<String> SpecialtyIds, Long courseId) {
		// 首先判断该专业是否有关联的课程
		List<CourseSpeciatlyRelationEntity> list = courseSpeciatlyRepo.findByCourseId(courseId);
		for (CourseSpeciatlyRelationEntity courseSpeciatly : list) {
			courseSpeciatlyRepo.delete(courseSpeciatly);
		}
		// 保存新关联的课程
		for (String specialtyId : SpecialtyIds) {
			CourseSpeciatlyRelationEntity courseSpeciatly = new CourseSpeciatlyRelationEntity();
			courseSpeciatly.setCourseId(courseId);
			courseSpeciatly.setSpecialtyId(Long.parseLong(specialtyId));
			courseSpeciatly.setCreator(userId);
			courseSpeciatlyRepo.save(courseSpeciatly);
		}

	}
}
