package cn.com.qmth.examcloud.core.basic.service.impl;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;
import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.exact;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.dao.ExamSiteRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.ExamSite;

@Service
public class ExamSiteService {

	@Autowired
	ExamSiteRepo examSiteRepo;

	public Page<ExamSite> findAll(ExamSite examSiteCriteria, Pageable pageable) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains())
				.withMatcher("code", contains()).withMatcher("orgId", exact());
		Example<ExamSite> examSiteExample = Example.of(examSiteCriteria, exampleMatcher);
		return examSiteRepo.findAll(examSiteExample, pageable);
	}

	public List<ExamSite> findAll(ExamSite examSiteCriteria) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains())
				.withMatcher("code", contains()).withMatcher("orgId", exact());
		Example<ExamSite> examSiteExample = Example.of(examSiteCriteria, exampleMatcher);
		return examSiteRepo.findAll(examSiteExample);
	}

	public ExamSite save(ExamSite examSite) {
		checkCode(examSite.getOrgId(), examSite.getCode());
		return examSiteRepo.save(examSite);
	}

	private void checkCode(Long orgId, String code) {
		ExamSite old = examSiteRepo.findFirstByOrgIdAndCode(orgId, code);
		if (old != null) {
			throw new StatusException("B-710002", "考点代码已存在");
		}
	}

	public ExamSite update(Long id, ExamSite examSite) {
		ExamSite old = examSiteRepo.findOne(id);
		if (!old.getCode().equals(examSite.getCode())) {
			checkCode(examSite.getOrgId(), examSite.getCode());
		}
		examSite.setUpdateTime(new Date());
		return examSiteRepo.save(examSite);
	}

}
