package cn.com.qmth.examcloud.core.basic.service.impl;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;

import java.io.InputStream;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.helpers.DynamicEnum;
import cn.com.qmth.examcloud.commons.base.helpers.DynamicEnumManager;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelError;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelReader;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelReaderHandle;
import cn.com.qmth.examcloud.commons.web.security.enums.RoleMeta;
import cn.com.qmth.examcloud.core.basic.base.constants.BasicConsts;
import cn.com.qmth.examcloud.core.basic.dao.ExamSiteRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgPropertyRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.ExamSite;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgPropertyEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.RoleEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRoleRelationEntity;
import cn.com.qmth.examcloud.core.basic.dao.enums.OrgProperty;
import cn.com.qmth.examcloud.core.basic.service.OrgService;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgDto;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgInfo;

@Service
public class OrgServiceImpl implements OrgService {

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	UserRepo userRepo;

	@Autowired
	ExamSiteRepo examSiteRepo;

	@Autowired
	UserRoleRelationRepo userRoleRelationRepo;

	@Autowired
	OrgPropertyRepo orgPropertyRepo;

	@Transactional
	public List<ExcelError> importLearnCenter(Long orgId, InputStream inputStream) {
		ExcelReader excelReader = new ExcelReader(OrgDto.class);
		List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {
			@Override
			public ExcelError handle(Object obj) {
				OrgDto dto = (OrgDto) obj;
				dto.setParentId(orgId);
				dto.setRootId(orgId);
				ExcelError error = importCheck(dto);
				if (error == null) {
					saveOrgAndExamSite(dto);
				}
				return error;
			}
		});
		return excelErrors;
	}

	/**
	 * 保存学习中心和考点
	 * 
	 * @param orgDto
	 */
	private void saveOrgAndExamSite(OrgDto orgDto) {
		OrgEntity org = orgRepo.findByParentIdAndCode(orgDto.getParentId(), orgDto.getCode());
		if (org == null) {
			OrgEntity tempOrg = orgRepo.save(orgAssembler(orgDto));
			orgDto.setId(tempOrg.getId());
			examSiteRepo.save(examSiteAssembler(orgDto));
			createLearnerCenterUser(tempOrg);
		} else {
			org.setName(orgDto.getName());
			org.setContacts(orgDto.getContacts());
			org.setTelephone(orgDto.getTelphone());
			orgRepo.save(org);

			orgDto.setId(org.getId());
			ExamSite examSite = examSiteRepo.findFirstByOrgIdAndCode(org.getId(),
					orgDto.getExamSiteCode());
			if (examSite == null) {
				examSiteRepo.save(examSiteAssembler(orgDto));
			} else {
				examSite.setName(orgDto.getExamSiteName());
				examSiteRepo.save(examSite);
			}
		}

	}

	/**
	 * 创建学习中心用户
	 * 
	 * @param org
	 */
	public void createLearnerCenterUser(OrgEntity org) {
		UserEntity user = userRepo.findByLoginName(org.getCode());
		if (null != user) {
			return;
		}
		user = new UserEntity();
		user.setRootOrgId(org.getRootId());
		user.setOrgId(org.getId());
		user.setLoginName(org.getCode());
		user.setName(org.getContacts());
		user.setPhoneNumber(org.getTelephone());
		user.setEnable(true);
		user.setPassword(BasicConsts.DEFAULT_PASSWORD);
		UserEntity saved = userRepo.save(user);
		List<UserRoleRelationEntity> userRoles = Lists.newArrayList();
		RoleEntity role = roleRepo.findByCode(RoleMeta.LC_USER.name());
		UserRoleRelationEntity relation = new UserRoleRelationEntity(saved.getId(), role.getId(),
				role.getCode());
		userRoles.add(relation);
		userRoleRelationRepo.save(userRoles);
	}

	private OrgEntity orgAssembler(OrgDto orgDto) {
		OrgEntity org = new OrgEntity();
		org.setRootId(orgDto.getRootId());
		org.setParentId(orgDto.getParentId());
		org.setCode(orgDto.getCode());
		org.setName(orgDto.getName());
		org.setContacts(orgDto.getContacts());
		org.setTelephone(orgDto.getTelphone());
		org.setEnable(true);
		return org;
	}

	private ExamSite examSiteAssembler(OrgDto orgDto) {
		ExamSite examSite = new ExamSite();
		examSite.setCode(orgDto.getExamSiteCode());
		examSite.setName(orgDto.getExamSiteName());
		examSite.setOrgId(orgDto.getId());
		examSite.setEnable(true);
		return examSite;
	}

	private ExcelError importCheck(OrgDto dto) {
		if (StringUtils.isBlank(dto.getCode())) {
			return new ExcelError("中心代码不能为空");
		}
		if (StringUtils.isBlank(dto.getName())) {
			return new ExcelError("中心名称不能为空");
		}
		if (StringUtils.isBlank(dto.getExamSiteCode())) {
			return new ExcelError("考点代码不能为空");
		}
		if (StringUtils.isBlank(dto.getExamSiteName())) {
			return new ExcelError("考点名称不能为空");
		}
		if (StringUtils.isBlank(dto.getContacts())) {
			return new ExcelError("联系人不能为空");
		}
		if (StringUtils.isBlank(dto.getTelphone())) {
			return new ExcelError("联系电话不能为空");
		}
		return null;
	}

	/**
	 * 启用/禁用机构；禁用机构时禁用该机构下的所有用户
	 * 
	 * @param id
	 * @param enable
	 * @return
	 */
	@Transactional
	public OrgEntity enableSchool(Long id, boolean enable) {
		OrgEntity org = orgRepo.findOne(id);
		org.setEnable(enable);
		orgRepo.save(org);
		return org;
	}

	public List<OrgEntity> findAll(OrgEntity orgCriteria) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains())
				.withMatcher("code", contains());
		Example<OrgEntity> examExample = Example.of(orgCriteria, exampleMatcher);
		return orgRepo.findAll(examExample);
	}

	/*
	 * 保存顶级机构
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.basic.service.OrgService#saveRootOrg(cn.com.
	 * qmth.examcloud.core.basic.service.bean.OrgInfo)
	 */
	@Override
	public OrgEntity saveRootOrg(OrgInfo orgInfo) {

		String code = orgInfo.getCode();
		String name = orgInfo.getName();
		Boolean enable = orgInfo.getEnable();
		String contacts = orgInfo.getContacts();
		String telephone = orgInfo.getTelephone();
		String domainName = orgInfo.getDomainName();
		String remark = orgInfo.getRemark();
		Map<String, String> properties = orgInfo.getProperties();

		if (StringUtils.isBlank(code)) {
			throw new StatusException("B-150001", "code is null");
		}
		if (StringUtils.isBlank(name)) {
			throw new StatusException("B-150002", "name is null");
		}

		if (null == enable) {
			enable = true;
		}

		OrgEntity orgEntity = orgRepo.findByParentIdIsNullAndCode(code);
		if (null == orgEntity) {
			OrgEntity tmp = new OrgEntity();
			tmp.setRootId(-1L);
			tmp.setCode(code);
			tmp.setEnable(enable);
			tmp.setName(name);
			orgEntity = orgRepo.save(tmp);
		} else {
			if (!orgEntity.getId().equals(orgEntity.getRootId())) {
				throw new StatusException("B-150003", "数据错误");
			}
		}

		orgEntity.setRootId(orgEntity.getId());
		orgEntity.setName(name);
		orgEntity.setEnable(enable);
		orgEntity.setContacts(contacts);
		orgEntity.setTelephone(telephone);
		orgEntity.setDomainName(domainName);
		orgEntity.setRemark(remark);

		OrgEntity saved = orgRepo.save(orgEntity);

		Map<DynamicEnum, String> map = checkAndGetOrgProperties(properties);
		for (Entry<DynamicEnum, String> entry : map.entrySet()) {
			DynamicEnum de = entry.getKey();
			String value = entry.getValue();
			OrgPropertyEntity entity = orgPropertyRepo.findByOrgIdAndKeyId(saved.getId(),
					de.getId());
			if (null == entity) {
				entity = new OrgPropertyEntity();
				entity.setOrgId(saved.getId());
				entity.setKeyId(de.getId());
			}
			entity.setValue(value);

			orgPropertyRepo.save(entity);
		}

		return saved;
	}

	/*
	 * 保存子机构
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.basic.service.OrgService#saveSubOrg(cn.com.
	 * qmth.examcloud.core.basic.service.bean.OrgInfo)
	 */
	@Override
	public OrgEntity saveSubOrg(OrgInfo orgInfo) {

		String code = orgInfo.getCode();
		String name = orgInfo.getName();
		Boolean enable = orgInfo.getEnable();
		String contacts = orgInfo.getContacts();
		String telephone = orgInfo.getTelephone();
		Long rootId = orgInfo.getRootId();
		Long parentId = orgInfo.getParentId();
		String remark = orgInfo.getRemark();
		Map<String, String> properties = orgInfo.getProperties();

		if (null == enable) {
			enable = true;
		}

		if (null == rootId) {
			throw new StatusException("B-150000", "rootId is null");
		}
		if (StringUtils.isBlank(code)) {
			throw new StatusException("B-150001", "code is null");
		}
		if (StringUtils.isBlank(name)) {
			throw new StatusException("B-150002", "name is null");
		}
		if (null == parentId) {
			throw new StatusException("B-150003", "parentId is null");
		}

		OrgEntity rootOrgEntity = orgRepo.findOne(rootId);
		if (null == rootOrgEntity) {
			throw new StatusException("B-150004", "rootId is wrong");
		}
		if (null != rootOrgEntity.getParentId()) {
			throw new StatusException("B-150005", "rootId is wrong");
		}

		OrgEntity parentOrgEntity = orgRepo.findOne(parentId);
		if (null == parentOrgEntity) {
			throw new StatusException("B-150006", "parentId is wrong");
		}

		if (!rootOrgEntity.getRootId().equals(parentOrgEntity.getRootId())) {
			throw new StatusException("B-150007", "parentId, rootId is wrong");
		}

		OrgEntity orgEntity = orgRepo.findByRootIdAndCode(rootId, code);
		if (null == orgEntity) {
			orgEntity = new OrgEntity();
			orgEntity.setRootId(rootId);
			orgEntity.setParentId(parentId);
		}

		orgEntity.setName(name);
		orgEntity.setEnable(enable);
		orgEntity.setContacts(contacts);
		orgEntity.setTelephone(telephone);
		orgEntity.setRemark(remark);

		OrgEntity saved = orgRepo.save(orgEntity);

		Map<DynamicEnum, String> map = checkAndGetOrgProperties(properties);
		for (Entry<DynamicEnum, String> entry : map.entrySet()) {
			DynamicEnum de = entry.getKey();
			String value = entry.getValue();
			OrgPropertyEntity entity = orgPropertyRepo.findByOrgIdAndKeyId(saved.getId(),
					de.getId());
			if (null == entity) {
				entity = new OrgPropertyEntity();
				entity.setOrgId(saved.getId());
				entity.setKeyId(de.getId());
			}
			entity.setValue(value);

			orgPropertyRepo.save(entity);
		}

		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param properties
	 * @return
	 */
	private Map<DynamicEnum, String> checkAndGetOrgProperties(Map<String, String> properties) {
		DynamicEnumManager manager = OrgProperty.getDynamicEnumManager();

		Map<DynamicEnum, String> map = Maps.newHashMap();
		if (null == properties) {
			return map;
		}
		for (Entry<String, String> entry : properties.entrySet()) {
			String key = entry.getKey();
			String value = entry.getValue();
			if (StringUtils.isBlank(value)) {
				continue;
			}
			DynamicEnum de = null;
			try {
				de = manager.getByName(key);
			} catch (Exception e) {
				throw new StatusException("E-001004", "机构属性错误");
			}
			map.put(de, value.trim());
		}

		return map;
	}

}
